/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package pedviz.loader;

import java.util.Vector;

import pedviz.graph.Graph;

/**
 * This abstract class is a super class for graph loader class. You can use this
 * guidelines for writing your own Loader class:
 * <ul>
 * <li>Overwrite the method load.
 * <li>Load all Nodes from your source and add them to the Graph object.</li>
 * <li>All informations about a indivual you must add to the Node with the
 * method setUserData().</li>
 * <li>If you have loaded all Nodes, then add all Edges to the Graph.</li>
 * </ul>
 * 
 * @author Lukas Forer
 * @version 0.1
 */
public abstract class Loader {
    private String idColumn = "id";

    private String momColumn = "mom";

    private String dadColumn = "dad";

    /**
     * Loads data in the given Graph object.
     * 
     * @param graph
     *                Gaph object.
     */
    public abstract void load(Graph graph);

    /**
     * Returns a array of all columns.
     * 
     * @return a array of all columns.
     */
    public abstract Vector<String> getColumns();

    /**
     * Sets the field names, in which the id, mom's id and dad's id are saved.
     * 
     * @param id
     *                field name from the id
     * @param mom
     *                field name from the mom's id
     * @param dad
     *                field name from the dad's id
     */
    public void setSettings(String id, String mom, String dad) {
	idColumn = id;
	momColumn = mom;
	dadColumn = dad;
    }

    /**
     * Returns the field name from the dad's id
     * 
     * @return field name from the dad's id
     */
    public String getDadColumn() {
	return dadColumn;
    }

    /**
     * Returns the field name from the id
     * 
     * @return field name from the id
     */
    public String getIdColumn() {
	return idColumn;
    }

    /**
     * Returns the field name from the mom's id
     * 
     * @return field name from the mom's id
     */
    public String getMomColumn() {
	return momColumn;
    }

}
