/* Copyright  2007 by Christian Fuchsberger and Lukas Forer info@pedvizapi.org.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License <http://www.pedvizapi.org/gpl.txt>
 * for more details. 
 */

package tutorials;

import javax.swing.JFrame;

import pedviz.algorithms.Sugiyama;
import pedviz.graph.*;
import pedviz.loader.CsvGraphLoader;
import pedviz.view.DefaultEdgeView;
import pedviz.view.GraphView2D;
import pedviz.view.rules.ShapeRule;
import pedviz.view.symbols.*;
import pedviz.view.*;
import java.io.*;
import javax.imageio.*;
import java.awt.geom.*;
import java.awt.*;
import java.awt.image.*;
import java.net.*;

class MyHaplotypeSymbol extends Symbol2D {

    private Image image = null;

    public MyHaplotypeSymbol(Image image) {
	this.image = image;
    }

    public void drawSymbol(Graphics2D g, Point2D.Float position, float size,
	    Color border, Color fill, NodeView node) {
	g.drawImage(image, (int) (position.x - (size / 2f)),
		(int) (position.y - (size / 2f)), (int) size, (int) size, null);
    }

    @Override
    public int getPriority() {
	return 0;
    }
}

public class Example3 {

    public static void main(String[] args) {

	// Step 1
	Graph graph = new Graph();
	CsvGraphLoader loader = new CsvGraphLoader("10081.pre", " ");
	loader.setSettings("Id", "Fid", "Mid");
	loader.load(graph);

	// Step 2
	DefaultNodeView nodeview = new DefaultNodeView();
	nodeview.addHintAttribute("Id");
	nodeview.addHintAttribute("Sex");
	DefaultEdgeView edgeview = new DefaultEdgeView();

	Sugiyama s = new Sugiyama(graph, nodeview, edgeview);
	s.run();

	// Creates a frame
	JFrame frame = new JFrame("Tutorials - Example 3");
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	frame.setSize(800, 600);

	// Step 3
	GraphView2D view = new GraphView2D(s.getLayoutedGraph());
	frame.add(view.getComponent());

	// Step 4
	try {
	    BufferedImage img = ImageIO.read(new File("man2.png"));
	    view.addRule(new ShapeRule("sex", "1", new MyHaplotypeSymbol(img)));	    
	} catch (Exception e) {
	    System.out.println(e);
	}
	try {
	    BufferedImage img = ImageIO.read(new File("woman2.png"));
	    view.addRule(new ShapeRule("sex", "2", new MyHaplotypeSymbol(img)));
	} catch (Exception e) {
	    System.out.println(e);
	}

	frame.setVisible(true);
    }
}
